/*
 * PAL RGB Colorbar Generator
 * Copyright (c) 2016 Marco Maccaferri
 *
 * 320x240 pixels
 * 64 colors, VGA mode
 *
 * Connections:
 *
 *        VGA     SCART
 * RED      1 ... 15   RED
 * GREEN    2 --- 11   GREEN
 * BLUE     3 ---  7   BLUE
 * HSYNC   13 --- 19   COMPOSITE OUT (F)
 *                        - or -
 *                20   COMPOSITE IN (TV)
 * VCC        --- 16   RGB SWITCH
 * GND        --- 5, 9, 13, 18
 *
 * MIT Licensed
 */

                        .pasm
                        .compress off

                        .section .cog_video_driver_320, "ax"

                        .equ    vpin, $0FF                  // pin group mask
                        .equ    vgrp, 2                     // pin group
                        .equ    hv_idle, $03030303          // h/v sync inactive
                        .equ    _3210, $E4                  // %%3210

                        .equ    res_x, 320                  // |
                        .equ    res_y, 240                  // UI support

                        .org     0

// Upset video h/w and relatives.

                        movi    ctra, #%0_00001_100         // PLL, VCO/8
                        mov     frqa, frqx                  // 7.5MHz * 16 / 8 = 15MHz

                        mov     vscl, #64                   // 1/64

                        movd    vcfg, #vgrp                 // pin group
                        movs    vcfg, #vpin                 // pins
                        movi    vcfg, #%0_01_1_00_000       // VGA, 4 colour mode

                        rdlong  cnt, #0
                        shr     cnt, #10                    // ~1ms
                        add     cnt, cnt
                        waitcnt cnt, #0                     // PLL needs to settle

                        mov     dira, mask                  // drive outputs

// Setup complete, enter display loop.

vsync
                        mov     vscl, pulse

                        mov     ecnt, #6
                        waitvid sync, pulse_short           // 6 short sync pulses (2uSec. / 30uSec.)
                        djnz    ecnt, #$-1

                        mov     ecnt, #5
                        waitvid sync, pulse_long            // 5 long sync pulses (30uSec. / 2uSec.)
                        djnz    ecnt, #$-1

                        mov     ecnt, #5
                        waitvid sync, pulse_short           // 5 short sync pulses (2uSec. / 30uSec.)
                        djnz    ecnt, #$-1

// Vertical sync chain done, do visible area.

                        mov     ecnt, #44
                        call    #blank                      // top border
                        djnz    ecnt, #$-1

                        mov     scnt, #res_y

_loop                                                       // 240 active lines
                        mov     vscl, hsync
                        waitvid sync, pulse_short           // 4uSec sync + 8uSec back porch + left border
                        
                        mov     vscl, hvis
                        movs    _vid, #pal+0

                        mov     ecnt, #res_x / 4
_vid                    mov     colors, 0-0
                        or      colors, idle
                        waitvid colors, #_3210              // send scanline
                        add     _vid, #1                    // advance src
                        djnz    ecnt, #_vid

                        mov     vscl, hback
                        waitvid idle, #0                    // right border

                        djnz    scnt, #_loop                // repeat for all lines

                        mov     ecnt, #28
                        call    #blank                      // bottom border
                        djnz    ecnt, #$-1

                        jmp     #vsync                      // next frame

blank
                        mov     vscl, hsync
                        waitvid sync, pulse_short
                        mov     vscl, hblank
                        waitvid idle, #0
blank_ret               ret

// initialised data and/or presets

idle                    long    hv_idle
sync                    long    (hv_idle ^ $0200) & $FFFF

pulse_short             long    $00000001                   // short sync pulse %%0000000000000001
pulse_long              long    $15555555                   // long sync pulse  %%0111111111111111
pulse                   long    30 << 12 | 480              // sync pulse 2us + 30us half line

hsync                   long    60 << 12 | (180 + 40)       // horizontal sync 4us + 8us back porch + left border
hvis                    long     2 << 12 | 8                // visible pixels
hback                   long     0 << 12 | 100              // right border

hblank                  long     0 << 12 | (960 - 180 - 40) // blank line

frqx                    long    402653184                   // 7.5MHz

mask                    long    vpin << (vgrp * 8)

// colour buffer

pal                     long    $FCFCFCFC, $FCFCFCFC, $FCFCFCFC, $FCFCFCFC, $FCFCFCFC, $FCFCFCFC, $FCFCFCFC, $FCFCFCFC, $FCFCFCFC, $FCFCFCFC
                        long    $F0F0F0F0, $F0F0F0F0, $F0F0F0F0, $F0F0F0F0, $F0F0F0F0, $F0F0F0F0, $F0F0F0F0, $F0F0F0F0, $F0F0F0F0, $F0F0F0F0
                        long    $3C3C3C3C, $3C3C3C3C, $3C3C3C3C, $3C3C3C3C, $3C3C3C3C, $3C3C3C3C, $3C3C3C3C, $3C3C3C3C, $3C3C3C3C, $3C3C3C3C
                        long    $30303030, $30303030, $30303030, $30303030, $30303030, $30303030, $30303030, $30303030, $30303030, $30303030
                        long    $CCCCCCCC, $CCCCCCCC, $CCCCCCCC, $CCCCCCCC, $CCCCCCCC, $CCCCCCCC, $CCCCCCCC, $CCCCCCCC, $CCCCCCCC, $CCCCCCCC
                        long    $C0C0C0C0, $C0C0C0C0, $C0C0C0C0, $C0C0C0C0, $C0C0C0C0, $C0C0C0C0, $C0C0C0C0, $C0C0C0C0, $C0C0C0C0, $C0C0C0C0
                        long    $0C0C0C0C, $0C0C0C0C, $0C0C0C0C, $0C0C0C0C, $0C0C0C0C, $0C0C0C0C, $0C0C0C0C, $0C0C0C0C, $0C0C0C0C, $0C0C0C0C
                        long    $54545454, $54545454, $54545454, $54545454, $54545454, $54545454, $54545454, $54545454, $54545454, $54545454

// uninitialised data and/or temporaries

colors                  res     1
ecnt                    res     1                           // element count

addr                    res     1                           // colour buffer reference
lcnt                    res     1                           // line counter
scnt                    res     1                           // scanlines

                        fit     $1F0
